<?php

namespace Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use \Teknisa\GER\Utilities\MessagesProcedureInflightor;
use Zeedhi\Framework\DTO\Response\Error;
use Doctrine\DBAL\Connection;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class GerarPlanoVooController extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName;
    protected $parameterBagColumns = array();
    protected $GerarPlanoVooProcedure;
    protected $GerarPlanoVooService;

    public function __construct($crudService, $parameterBag,  $environment, $dataSourceManager, $GerarPlanoVooProcedure, $GerarPlanoVooProcedureGerados, $GerarPlanoVooProcedureCancelado, $GerarPlanoVooService, $MessagesProcedureInflightor, $entityManager) {
        $this->crudService = $crudService;
        $this->parameterBag=$parameterBag;
        $this->environment=$environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->GerarPlanoVooProcedure = $GerarPlanoVooProcedure;
        $this->GerarPlanoVooProcedureGerados = $GerarPlanoVooProcedureGerados;
        $this->GerarPlanoVooProcedureCancelado = $GerarPlanoVooProcedureCancelado;
        $this->GerarPlanoVooService = $GerarPlanoVooService;
        $this->MessagesProcedureInflightor = $MessagesProcedureInflightor;
        $this->entityManager = $entityManager;
    }

    //Find para os Voos Disponiveis/Selecionados e Classes
    public function findVooSelected(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'gerar_plano_voo_selecionados';
        $this->parameterBagColumns = array("CDFILIAL", "DATA", "ORDEM", "NRORG");
        parent::find($request, $response);
    }
    public function findVooDisponivel(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'gerar_plano_voo_disponiveis';
        $this->parameterBagColumns = array("CDFILIAL", "DTDECOLVOO", "ORDEM", "NRORG");
        parent::find($request, $response);
    }
    public function findClasse(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'gerar_plano_voo_classe';
        $this->parameterBagColumns = array("CDFILIAL", "CDCIAPPT", "NRVOOPPT", "CDTRECHOPPT", "DTHRPLAPPT", "NRORG");
        parent::find($request, $response);
    }
    public function findVooGerado(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'gerar_plano_voo_gerados';
        $this->parameterBagColumns = array("CDFILIAL", "DTHRPLAVOOPV", "NRORG");
        parent::find($request, $response);
    }
    public function findVooAtendido(DTO\Request\filter $request, DTO\Response $response){
        $this->dataSourceName = 'gerar_plano_voo_atendidos';
        $this->parameterBagColumns = array("CDFILIAL", "DTHRPLAVOOPV", "NRORG");
        parent::find($request, $response);
    }
    public function findEquip(DTO\Request\filter $request, DTO\Response $response){
        $this->dataSourceName = 'gerar_plano_voo_equipamento';
        $this->parameterBagColumns = array("CDFILIAL", "CDCIA", "NRORG");
        parent::find($request, $response);
    }
    private function selectClasse($row, DTO\Response $response){
        $this->dataSourceName = 'gerar_plano_voo_classe';
        $condition = array(
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $row["P_CDFILIAL"]
            ),
            array(
                "columnName" => "CDCIAPPT",
                "operator" => "=",
                "value" => $row["P_CDCIA"]
            ),
            array(
                "columnName" => "NRVOOPPT",
                "operator" => "=",
                "value" => $row["P_NRVOO"]
            ),
            array(
                "columnName" => "CDTRECHOPPT",
                "operator" => "=",
                "value" => $row["P_CDTRECHO"]
            ),
            array(
                "columnName" => "DTHRPLAPPT",
                "operator" => "=",
                "value" => $row["P_DTHRPLAVOOPV"]
            )
        );
        $filterCriteria = new FilterCriteria($this->dataSourceName, $condition);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        $registrosEncontrados = $dataSet->getRows();
        if(!empty($registrosEncontrados)){
            return $registrosEncontrados;
        }
        else {
            $this->dataSourceName = 'voo';
            $conditionVoo = array(
                array(
                    "columnName" => "CDFILIAL",
                    "operator" => "=",
                    "value" => $row["P_CDFILIAL"]
                ),
                array(
                    "columnName" => "CDCIA",
                    "operator" => "=",
                    "value" => $row["P_CDCIA"]
                ),
                array(
                    "columnName" => "NRVOO",
                    "operator" => "=",
                    "value" => $row["P_NRVOO"]
                )
            );
            $filterCriteria = new FilterCriteria($this->dataSourceName, $conditionVoo);
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
            $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
            $classe = $dataSet->getRows();
            if(!empty($classe)){
                return $classe;
            } else {
                $response->setError(new Error("ATE21100_CLASS_N_ENCONTRADA", 500));
            }
        }
    }

    //Monta os parametros e executa a Procedure GERA_PREVISAO_PAX
    public function executeProcedurePrevisao(DTO\Request\DataSet $request, DTO\Response $response){

        $rows = $request->getDataSet()->getRows();
        foreach ($rows as $row) {
            try {
                $this->GerarPlanoVooProcedure->executeProcedure($row['P_CDFILIAL'],$row['P_DTRELPLANPAX'],$row['P_NRRELPLANPAX'],$row['P_CDCIA'],$row['P_NRVOO'],$row['P_CDTRECHO'], $row['P_DTHRPLAVOOPV'],$row['P_TIPPREVISAO'],$row['P_IDTPGERPPT'],$row['P_VRPEROCUPACAO'],$row['P_SIMULACAO'],$row['P_NRDIAS'],$row['P_CDEQUIP'],$row['P_SGPREFIXO'], $this->environment->getCdOperador(), $this->environment->getNrOrgTrab());
            } catch (\Exception $e) {
                $response->setError(new Error($this->MessagesProcedureInflightor->cleanMessage($e->getMessage()), 500));
            }
        }
    }

    /* Chama a função selectClasse para receber o CDEQUIP da classe
        e executa a procedure GERA_PREVISAO_PAX
        utilizada para atualizar as classes de acordo com as opções de
        P_TIPPREVISAO : 1 - Cria as classes e associa elas ao Voo Selecionado
                        2 - Altera a quantidade de passageiros de acordo com o Histórico
                        3 - Altera a quantidade de passageiros de acordo com o Máximo permitido
                        4 - OPÇÃO DESABILITADA
                        5 - Altera a quantidade de passageiros de acordo com a Media dos dias informados
    */
    public function executeProcedureAlterClassCancelados(DTO\Request\DataSet $request, DTO\Response $response){
        $rows = $request->getDataSet()->getRows();
        $connection = $this->entityManager->getConnection();
        $arrayErros = array();  
            try {
                $connection->beginTransaction();
                foreach ($rows as $row) {
                    $classe = $this->selectClasse($row, $response);
                    try {
                        if(empty($classe[0]['CDEQUIPPPT'])){
                            $classe[0]['CDEQUIPPPT'] = $classe[0]['CDEQUIP'];
                            $this->GerarPlanoVooProcedure->executeProcedure($row['P_CDFILIAL'],$row['P_DTRELPLANPAX'],$row['P_NRRELPLANPAX'],$row['P_CDCIA'],$row['P_NRVOO'],$row['P_CDTRECHO'], $row['P_DTHRPLAVOOPV'], '1',$row['P_IDTPGERPPT'],$row['P_VRPEROCUPACAO'],$row['P_SIMULACAO'],$row['P_NRDIAS'],$classe[0]['CDEQUIPPPT'],$row['P_SGPREFIXO'], $this->environment->getCdOperador(), $this->environment->getNrOrgTrab());
                        }
                        $this->newGeraPlanoVooProcedure($row, $classe[0]['CDEQUIPPPT']);
                        $this->executeProcedureGerados($request, $response);
                    } catch(\Exception $ei){
                        array_push($arrayErros, $this->MessagesProcedureInflightor->cleanMessage($ei->getMessage()));
                    }
                }
                $connection->commit();
                if ($arrayErros !== []) {
                    $response->addDataSet(new DataSet("ListaErros", $arrayErros));
                }
            } catch (\Exception $e){
                $connection->rollback();
                $response->setError(new Error($this->MessagesProcedureInflightor->cleanMessage($e->getMessage()), 500));
            }
    }
    
    public function executeProcedureAlterClass(DTO\Request\DataSet $request, DTO\Response $response){
        $rows = $request->getDataSet()->getRows();
        $connection = $this->entityManager->getConnection();
        $arrayErros = array();  
            try {
                $connection->beginTransaction();
                foreach ($rows as $row) {
                    $classe = $this->selectClasse($row, $response);
                    try {
                        if(empty($classe[0]['CDEQUIPPPT'])){
                            $classe[0]['CDEQUIPPPT'] = $classe[0]['CDEQUIP'];
                            $this->GerarPlanoVooProcedure->executeProcedure($row['P_CDFILIAL'],$row['P_DTRELPLANPAX'],$row['P_NRRELPLANPAX'],$row['P_CDCIA'],$row['P_NRVOO'],$row['P_CDTRECHO'], $row['P_DTHRPLAVOOPV'], '1',$row['P_IDTPGERPPT'],$row['P_VRPEROCUPACAO'],$row['P_SIMULACAO'],$row['P_NRDIAS'],$classe[0]['CDEQUIPPPT'],$row['P_SGPREFIXO'], $this->environment->getCdOperador(), $this->environment->getNrOrgTrab());
                        }
                        $this->newGeraPlanoVooProcedure($row, $classe[0]['CDEQUIPPPT']);
                    } catch(\Exception $ei){
                        array_push($arrayErros, $this->MessagesProcedureInflightor->cleanMessage($ei->getMessage()));
                    }
                }
                $connection->commit();
                if ($arrayErros !== []) {
                    $response->addDataSet(new DataSet("ListaErros", $arrayErros));
                }
            } catch (\Exception $e){
                $connection->rollback();
                $response->setError(new Error($this->MessagesProcedureInflightor->cleanMessage($e->getMessage()), 500));
            }
    }
    
    public function newGeraPlanoVooProcedure($row, $equip){
        $connection = $this->entityManager->getConnection();
        $procedure = new StoredProcedure($connection, 'GERA_PREVISAO_PAXM');
        
        $procedure->addParam(new Param('P_CDFILIAL',      Param::PARAM_INPUT, $row['P_CDFILIAL']));
		$procedure->addParam(new Param('P_DTRELPLANPAX',  Param::PARAM_INPUT, $row['P_DTRELPLANPAX']));
		$procedure->addParam(new Param('P_NRRELPLANPAX',  Param::PARAM_INPUT, $row['P_NRRELPLANPAX']));
		$procedure->addParam(new Param('P_CDCIA',         Param::PARAM_INPUT, $row['P_CDCIA']));
		$procedure->addParam(new Param('P_NRVOO',         Param::PARAM_INPUT, $row['P_NRVOO']));
		$procedure->addParam(new Param('P_CDTRECHO',      Param::PARAM_INPUT, $row['P_CDTRECHO']));
		$procedure->addParam(new Param('P_DTHRPLAVOOPV',  Param::PARAM_INPUT, $row['P_DTHRPLAVOOPV']));
		$procedure->addParam(new Param('P_TIPPREVISAO',   Param::PARAM_INPUT, $row['P_TIPPREVISAO']));
		$procedure->addParam(new Param('P_IDTPGERPPT',    Param::PARAM_INPUT, $row['P_IDTPGERPPT']));
		$procedure->addParam(new Param('P_VRPEROCUPACAO', Param::PARAM_INPUT, $row['P_VRPEROCUPACAO']));
		$procedure->addParam(new Param('P_SIMULACAO',     Param::PARAM_INPUT, $row['P_SIMULACAO']));
		$procedure->addParam(new Param('P_NRDIAS',        Param::PARAM_INPUT, $row['P_NRDIAS']));
		$procedure->addParam(new Param('P_CDEQUIP',       Param::PARAM_INPUT, $equip));
		$procedure->addParam(new Param('P_SGPREFIXO',     Param::PARAM_INPUT, $row['P_SGPREFIXO']));
		$procedure->addParam(new Param('P_CDOPERADOR',    Param::PARAM_INPUT, $this->environment->getCdOperador()));
		$procedure->addParam(new Param('P_SGPREFIXO',     Param::PARAM_INPUT, $this->environment->getNrOrgTrab()));
	    $procedure->execute();
    }
    
    public function executeProcedureInitClass(DTO\Request\DataSet $request, DTO\Response $response){
        $rows = $request->getDataSet()->getRows();
            try {
                foreach ($rows as $row) {
                    $this->GerarPlanoVooProcedure->executeProcedure($row['P_CDFILIAL'],$row['P_DTRELPLANPAX'],$row['P_NRRELPLANPAX'],$row['P_CDCIA'],$row['P_NRVOO'],$row['P_CDTRECHO'], $row['P_DTHRPLAVOOPV'], $row['P_TIPPREVISAO'],$row['P_IDTPGERPPT'],$row['P_VRPEROCUPACAO'],$row['P_SIMULACAO'],$row['P_NRDIAS'],null,$row['P_SGPREFIXO'], $this->environment->getCdOperador(), $this->environment->getNrOrgTrab());
                }
            } catch (\Exception $e){
                $response->setError(new Error($this->MessagesProcedureInflightor->cleanMessage($e->getMessage()), 500));
            }
    }
    public function executeProcedureAlterEquip(DTO\Request\DataSet $request, DTO\Response $response){
        $rows = $request->getDataSet()->getRows();
        try {
            $this->dataSourceName = 'prevpaxtempm';
            parent::delete($request, $response);
            $moment = new \DateTime();
            $this->configuraEquipamento($request, $response);
            $rows[0]['CDEQUIPPPT'] = $rows[0]['CDALTER'];
            $this->GerarPlanoVooProcedure->executeProcedure($rows[0]['CDFILIAL'], $moment->format("d/m/Y"), null, $rows[0]['CDCIAPPT'], $rows[0]['NRVOOPPT'], $rows[0]['CDTRECHOPPT'], $rows[0]['DTHRPLAPPT'], '1', 'P', null, '9', 0, $rows[0]['CDEQUIPPPT'], null, $this->environment->getCdOperador(), $this->environment->getNrOrgTrab());
        } catch (\Exception $e){
            $response->setError(new Error($this->MessagesProcedureInflightor->cleanMessage($e->getMessage()), 500));
        }
    }

    private function configuraEquipamento(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = 'planovooequip';
        $rowsEquip = $this->getRowsToSaveEquip($request);
        $newRequest = $this->createRequestEquipamento($rowsEquip, false);
        parent::delete($newRequest, $response);
        $requestSave = $this->createRequestEquipamento($rowsEquip, true);
        parent::save($requestSave, $response);
    }

    private function getRowsToSaveEquip(DTO\Request\DataSet $request){
        $dataSet = $request->getDataSet()->getRows();
        $dataSave = array();
        $rowSave = array(
            "CDFILIAL"     => $dataSet[0]['CDFILIAL'],
            "CDCIA"        => $dataSet[0]['CDCIAPPT'],
            "NRVOO"        => $dataSet[0]['NRVOOPPT'],
            "CDTRECHO"     => $dataSet[0]['CDTRECHOPPT'],
            "DTHRPLAVOOPV" => $dataSet[0]['DTHRPLAPPT'],
            "CDEQUIP"      => $dataSet[0]['CDALTER'],
            "changed"      => true
        );
        array_push($dataSave, $rowSave);
        return $dataSave;
    }

    private function createRequestEquipamento($rows, $isNew){
        foreach ($rows as &$row) {
            $row['__is_new'] = $isNew;
        }
        $dataSet = new DataSet('planovooequip', $rows);
        $newRequest = new DTO\Request\DataSet($dataSet, '','','');
        if($isNew == true){
            $rowsSave = $this->crudService->getRowsToSave($newRequest, 'planovooequip');
            $dataSet = new DataSet('planovooequip', $rowsSave);
            return new DTO\Request\DataSet($dataSet, '','','');
        } else {
            return $newRequest;
        }
    }

    public function executeProcedureGerados(DTO\Request\DataSet $request, DTO\Response $response){
        $rows = $request->getDataSet()->getRows();
        $connection = $this->entityManager->getConnection();
        $arrayErros = [];
        try {
            $connection->beginTransaction();
            foreach ($rows as $row) {
                $row['P_CDOPERADOR'] = $this->environment->getCdOperador();
                $row['P_CDFILIPROC'] = $this->environment->getCdfilial();
                $row['P_NRORG'] = $this->environment->getNrorg();
                $resultado = $this->GerarPlanoVooProcedureGerados->executeProcedure($row['P_CDFILIPROC'],$row['P_CDOPERADOR'], "3", $row['P_CDFILIAL'], $row['P_CDCIA'], $row['P_NRVOO'], $row['P_CDTRECHO'], $row['P_DTHRPLAVOOPV'], $row['P_NRORG']);

                if($resultado['P_NRLISTAERROS'] != null) {
                    $arrayErros = $this->GerarPlanoVooService->concatErrorMessages($row['P_CDFILIPROC'], $resultado['P_NRLISTAERROS'], $arrayErros, $row['CIAVOO']);
                }
                if($row['P_TIPOOPER'] == 1){
                    $this->GerarPlanoVooProcedureCancelado->executeProcedure($row['P_CDFILIAL'], $row['P_CDCIA'], $row['P_NRVOO'], $row['P_CDTRECHO'], $row['P_DTHRPLAVOOPV'], $row['P_TIPOOPER'], $this->environment);
                }
            }

            $connection->commit();
            if ($arrayErros !== []) {
                $response->addDataSet(new DataSet("ListaErros", $arrayErros));
            }
        } catch (\Exception $e){
            $connection->rollback();
            $response->setError(new Error($this->MessagesProcedureInflightor->cleanMessage($e->getMessage()), 500));
        }
    }

    public function saveClasseInlineEdit($request, $response){
        $rows = $request->getDataSet()->getRows();
        $errorMessages = array();
        $rowsToSave = array();
        try{
            foreach($rows as $classe){
                $classe = (array)$classe;
                $classeMaxima = $this->getMaximaClasse($classe);

                $maximaTotal = $classeMaxima['QTPAX'] + $classeMaxima['QTPAXSBY'];
                $totalClasse = $classe['QTTOTPAXATPPT'] + $classe['QTPAXSBYPPT'];
                if($maximaTotal < $totalClasse){
                    $msg = $classe['SGCLASSE'].'#_#'.$classeMaxima['QTPAX'].'#_#'.$classeMaxima['QTPAXSBY'];
                    $errorMessages[] = $msg;
                } else {
                    $rowsToSave[] = $classe;
                }
            }
            $rowsToSave = $this->mountRowsToSaveClasse($rowsToSave);
            $this->crudService->save('prevpaxtempm', $rowsToSave);
            $response->addDataset(new DataSet('listaErros', $errorMessages));
        } catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    private function getMaximaClasse($row){
        $conditions = array(
            array(
                'columnName' => 'CDFILIAL',
                'operator' => '=',
                'value' => $row['CDFILIAL']
            ),
            array(
                'columnName' => 'CDCIA',
                'operator' => '=',
                'value' => $row['CDCIAPPT']
            ),
            array(
                'columnName' => 'CDTRECHO',
                'operator' => '=',
                'value' => $row['CDTRECHOPPT']
            ),
            array(
                'columnName' => 'NRVOO',
                'operator' => '=',
                'value' => $row['NRVOOPPT']
            ),
            array(
                'columnName' => 'CDCLASSE',
                'operator' => '=',
                'value' => $row['CDCLASSEPPT']
            )
        );
        $dataSet = $this->crudService->getFilteredDataSet('config_pax_voo', $conditions);
        return $dataSet->getRows()[0];
    }

    private function mountRowsToSaveClasse($rows){
        $dataSet = new DataSet('prevpaxtempm', $rows);
        $newRequest = new DTO\Request\DataSet($dataSet, '','','');
        return $this->crudService->getRowsToSave($newRequest, 'prevpaxtempm');
    }
}